<?php
// $Id:
/**
 * Argument handler to accept proximity
 */
class location_handler_argument_location_distance extends views_handler_argument {
  /**
   * Default value options.
  */
  function option_definition() {
    $options = parent::option_definition();
    $options['operator'] = array('default' => 'dist');
    $options['search_units'] = array('default' => 'mile');
    $options['default_distance'] = array('default' => '100');
    $options['type'] = array('default' => 'postal');
    $options['separator'] = array('default' => '_');
    $options['value'] = array(
      'default' => array(
        'latitude' => '',
        'longitude' => '',
        'postal_code' => '',
        'city' => '',
        'country' => variable_get('location_default_country', 'us'),
      ),
    );
    return $options;
  }
  function operator_options() {
    return array(
      'mbr' => t('Proximity (Rectangular)'),
      'dist' => t('Proximity (Circular)'),
    );
  }

  /**
   * Add a form elements to select options for this argument.
   */
  function options_form(&$form, &$form_state) {
    parent::options_form($form, $form_state);
    $options = array('postal' => t('Postal Code (Zipcode)'), 
                     'postal_or_city' => t('Postal Code  (Zipcode) or City'),
                     'city' => t('City'),
                     'latlon' => t('Decimal Lattitude and Longitude coordinates'),
                     );
    $form['type'] = array(
      '#title' => t('Coordinate Description'),
      '#type' => 'radios',
      '#options' => $options,
      '#default_value' => $this->options['type'],
      '#description' => t('Type of center point'),
     );
    $form['separator'] = array(
      '#title' => t('Separator'),
      '#type' => 'textfield',
      '#default_value' => $this->options['separator'],
      '#description' => t('Separator for coordinates and distance.  Must not occur in the postal code.'),
    ); 
    //default parameter
    $form['default_distance'] = array(
      '#weight' => -48,
      '#type' => 'textfield',
      '#title' => t('Default Distance'),
      '#default_value' => $this->options['default_distance'],
      '#description' => t('When 3rd number is ommitted'),
    );
    $options = array('mile' => 'Miles', 'km' => 'Kilometers');
    $form['search_units'] = array(
      '#title' => t('Units'),
      '#type' => 'radios',
      '#options' => $options,
      '#default_value' => $this->options['search_units'],
      '#multiple' => TRUE,
      '#description' => t("Select the units of distance"),
    );      
	$form['operator'] = array(
      '#type' => 'select',
      '#title' => t('Proximity type'),
      '#weight' => -47,
      '#description' => t('Square (faster) or circle (more accurate)?'),
      '#default_value' => $this->options['operator'],
      '#options' => array('mbr' => t('Square'),'dist' => t('Circular'))
   );
  } 

  
  function options_validate($form, &$form_state) {
    // It is very important to call the parent function here:
    parent::options_validate($form, $form_state);
    // must escape * + ? . ( ) [ ] { } / | ^ $ in regex
    if (preg_match('[\$|&|\+|,|\/|:|;|=|\?|@|#]', $form_state['values']['options']['separator'])) {
       form_error($form['separator'], t('The characters $ & + , / : ; = ? @ # are reserved or unsafe and must be encoded'));	
    }
  }  
  
  // Used from the distance field.
  function calculate_coords() {
    if (!empty($this->value['latitude']) && !empty($this->value['longitude'])) {
      // If there are already coordinates, there's no work for us.
      return TRUE;
    }
    // @@@ Switch to mock location object and rely on location more?

    if (in_array($this->options['type'], array('postal', 'city', 'postal_or_city'))) {
      // Force default for country.
      $this->value['country'] = variable_get('location_default_country', 'us');
      $coord = NULL;
      // Zip code lookup.
      if (!empty($this->value['postal_code']) && in_array($this->options['type'], array('postal', 'postal_or_city'))) {
        $coord = location_get_postalcode_data($this->value, TRUE);
        //will fail validation if not a proper postal code and $coord will be NULL
      }
      // City lookup using postcode value 
      if (!$coord && empty($this->value['city']) && !empty($this->value['postal_code']) && in_array($this->options['type'], array('postal_or_city'))) {
        //copy the value of postal_code into city
        $this->value['city'] = $this->value['postal_code'];
        unset($this->value['postal_code']);
        $coord = location_get_city_data($this->value, TRUE);
        // clean up by setting the values back and returning false;
        if (!$coord){
          $this->value['postal_code'] = $this->value['city'];
          unset($this->value['city']);
          return false;
        }
      }
      // City lookup. 
      if (!empty($this->value['city']) && in_array($this->options['type'], array('city'))) {
        $coord = location_get_city_data($this->value, TRUE);
      }
      if ($coord) {
        $this->value['latitude'] = $coord['lat'];
        $this->value['longitude'] = $coord['lon'];
        // Add city, post_code and province from our results to $this->value,
        // which keeps things accurate and tidy
        if (array_key_exists('city', $coord) && !empty($coord['city'])) {
          $this->value['city'] = $coord['city'];
        }
        if (array_key_exists('postal_code', $coord) && !empty($coord['postal_code'])) {
          $this->value['postal_code'] = $coord['postal_code'];
        }
        if (array_key_exists('province', $coord) && !empty($coord['province'])) {
          $this->value['province'] = $coord['province'];
        }
      }
      else {
        return false;
      }
    }
    else {
        return false;
    }
    if (empty($this->value['latitude']) || empty($this->value['longitude'])) {
      return false;
    }
    return true;
  }

  function query() {
    //set default values
    /*
    $this->value = array(
      'latitude' => '',
      'longitude' => '',
      'postal_code' => '',
      'city' => '',
      'country' => variable_get('location_default_country', 'us'),
      'province' => '',
      
    );*/
    $this->value['search_units'] = $this->options['search_units'];

  	//get and process argument
	foreach ($this->view->argument as $argument) {
	  if ($argument->field == 'distance') {
	    $split_args = split($this->options['separator'], $this->view->args[$argument->position]);
	    break;
	  }
	}
  	if ($this->options['type'] == 'postal' || $this->options['type'] == 'postal_or_city') {
	  $this->value['postal_code'] = $split_args[0];
	  $this->value['search_distance'] = (count($split_args)>1)? $split_args[1] : $this->options['default_distance'];

  	} else if ($this->options['type'] == 'city') {
	  $this->value['city'] = $split_args[0];
	  $this->value['search_distance'] = (count($split_args)>1)? $split_args[1] : $this->options['default_distance'];

  	} else if ($this->options['type'] == 'latlon') {
  	  list($this->value['latitude'], $this->value['longitude']) = split(',', $split_args[0]);
	  $this->value['search_distance'] = (count($split_args)>1)? $split_args[1] : $this->options['default_distance'];
  	}
  	
    /*****FIX BEFORE COMMIT ****/
    //what is group and why do I need to set this to work??
    $this->options['group'] = 0;
    // Coordinates available?
    if (!$this->calculate_coords()) {
      // Distance set?
      if (!empty($this->value['search_distance'])) {
        // Hmm, distance set but unable to resolve coordinates.
        // Force nothing to match.
        $this->query->add_where($this->options['group'], "0");
      }
      return;
    }

    $this->ensure_my_table();

    $lat = $this->value['latitude'];
    $lon = $this->value['longitude'];

    $distance_meters = _location_convert_distance_to_meters($this->value['search_distance'], $this->value['search_units']);

    $latrange = earth_latitude_range($lon, $lat, $distance_meters);
    $lonrange = earth_longitude_range($lon, $lat, $distance_meters);

    // Add MBR check (always.)
    $this->query->add_where($this->options['group'], "$this->table_alias.latitude > %f AND $this->table_alias.latitude < %f AND $this->table_alias.longitude > %f AND $this->table_alias.longitude < %f", $latrange[0], $latrange[1], $lonrange[0], $lonrange[1]);

    if ($this->options['operator'] == 'dist') {
      // Add radius check.
      $this->query->add_where($this->options['group'], earth_distance_sql($lon, $lat, $this->table_alias) .' < %f', $distance_meters);
    }
  }
}